package detector.movement;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.maxmind.geoip.Location;
import com.maxmind.geoip.LookupService;

import detector.Entry;
import detector.EntryStorage;

/**
 * @author Roman Tekhov
 */
public class MovementCalculator {
	
	private EntryStorage entryStorage;
	private LookupService lookupService;
	
	
	public MovementCalculator(EntryStorage entryStorage, LookupService lookupService) {
		this.entryStorage = entryStorage;
		this.lookupService = lookupService;
	}
	
	
	void calculate(MovementProcessor processor) {				
		Map<String, Collection<Entry>> entriesByUser = entryStorage.getEntriesByUser();
		
		for(Map.Entry<String, Collection<Entry>> mapEntry : entriesByUser.entrySet()) {
			List<Movement> movements = new ArrayList<Movement>();
			
			Entry entryFrom = null;
			Location locationFrom = null;
			
			for(Entry entry : mapEntry.getValue()) {
				if(entryFrom != null && !entryFrom.getHost().equals(entry.getHost())) {
					Location locationTo = lookupService.getLocation(entry.getHost());
					
					if(locationFrom != null && locationTo != null) {
						entryFrom.setLocation(locationFrom);
						entry.setLocation(locationTo);
						
						movements.add(new Movement(entryFrom, entry));
					}					
					
					locationFrom = locationTo;
					
				} else if(entryFrom == null) {
					locationFrom = lookupService.getLocation(entry.getHost());
				}
				
				entryFrom = entry;
			}
			
			if(!movements.isEmpty()) {
				processor.process(new UserMovements(mapEntry.getKey(), movements));
			}
		}
	}
	

	interface MovementProcessor {
		
		void process(UserMovements movements);
	}


}
